"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PorticoApi = exports.OKU_TRADE_BASE_URL = exports.RELAYER_FEE_API_URL = void 0;
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const axios_1 = __importDefault(require("axios"));
const abis_1 = require("./abis");
const utils_1 = require("./utils");
exports.RELAYER_FEE_API_URL = 'https://gfx.relayers.xlabs.xyz/api/v1/swap/quote';
exports.OKU_TRADE_BASE_URL = 'https://oku.trade/app';
const CREATE_ORDER_API_URL = 'https://thermae.fly.dev/api/order/create';
class PorticoApi {
    // Post the order to the portico API
    static async createOrder(request) {
        try {
            const response = await axios_1.default.post(CREATE_ORDER_API_URL, request);
            // Validate the response, not strictly necessary but if some details are wrong we want to know
            this.validateCreateOrderResponse(response.data, request);
            return response.data;
        }
        catch (e) {
            if (axios_1.default.isAxiosError(e)) {
                const message = `${e.response?.statusText}: ${e.response?.data.message}`;
                throw new Error(`Could not create order: ${message},`);
            }
            throw e;
        }
    }
    /**
     * Validates that the response from the order creation API matches the request
     * throws an error if there is a mismatch
     */
    static validateCreateOrderResponse = (response, request) => {
        if (!(0, utils_1.isEqualCaseInsensitive)(request.porticoAddress || '', response.transactionTarget)) {
            throw new Error('portico address mismatch');
        }
        const decoded = abis_1.porticoAbi.decodeFunctionData('start', response.transactionData);
        if (decoded.length !== 1 || decoded[0].length !== 10) {
            throw new Error('decoded length mismatch');
        }
        const flagSetBuffer = Buffer.from(decoded[0][0].slice(2), 'hex');
        if (flagSetBuffer.length !== 32) {
            throw new Error('flag set length mismatch');
        }
        const { recipientChain, feeTierStart, feeTierFinish, shouldWrapNative, shouldUnwrapNative,
        //@ts-ignore
         } = parseFlagSet(flagSetBuffer);
        if (recipientChain !== request.destinationChainId) {
            throw new Error('recipient chain mismatch');
        }
        if (feeTierStart !== request.feeTierStart) {
            throw new Error('fee tier start mismatch');
        }
        if (feeTierFinish !== request.feeTierEnd) {
            throw new Error('fee tier end mismatch');
        }
        if (shouldWrapNative !== request.shouldWrapNative) {
            throw new Error('should wrap native mismatch');
        }
        if (shouldUnwrapNative !== request.shouldUnwrapNative) {
            throw new Error('should unwrap native mismatch');
        }
        const startTokenAddress = decoded[0][1];
        if (!(0, utils_1.isEqualCaseInsensitive)(startTokenAddress, request.startingToken)) {
            throw new Error('start token address mismatch');
        }
        // const startTokenId = Wormhole.chainAddress(toChain(request.startingChainId), startTokenAddress);
        // const canonicalTokenAddress: string = decoded[0][2];
        // if (!isEqualCaseInsensitive(canonicalTokenAddress, getOriginalAddress("Mainnet", startTokenId))) {
        //   throw new Error("canonical token address mismatch");
        // }
        const finalTokenAddress = decoded[0][3];
        if (!(0, utils_1.isEqualCaseInsensitive)(finalTokenAddress, request.destinationToken)) {
            throw new Error('final token address mismatch');
        }
        const recipientAddress = decoded[0][4];
        if (!(0, utils_1.isEqualCaseInsensitive)(recipientAddress, request.destinationAddress)) {
            throw new Error('recipient address mismatch');
        }
        const destinationPorticoAddress = decoded[0][5];
        if (!(0, utils_1.isEqualCaseInsensitive)(destinationPorticoAddress, request.destinationPorticoAddress || '')) {
            throw new Error('destination portico address mismatch');
        }
        const amountSpecified = decoded[0][6];
        if (amountSpecified.toString() !== request.startingTokenAmount) {
            throw new Error('amount mismatch');
        }
        const minAmountStart = decoded[0][7];
        if (minAmountStart.toString() !== request.minAmountStart) {
            throw new Error('min amount start mismatch');
        }
        const minAmountFinish = decoded[0][8];
        if (minAmountFinish.toString() !== request.minAmountEnd) {
            throw new Error('min amount finish mismatch');
        }
        const relayerFee = decoded[0][9];
        if (relayerFee.toString() !== request.relayerFee) {
            throw new Error('relayer fee mismatch');
        }
    };
    static async quoteRelayer(from, to) {
        const targetChain = (0, connect_sdk_1.toChainId)(to.chain);
        const sourceToken = connect_sdk_1.encoding.hex.encode(from.address.toUniversalAddress().toUint8Array(), false);
        const targetToken = connect_sdk_1.encoding.hex.encode(to.address.toUniversalAddress().toUint8Array(), false);
        const request = { targetChain, sourceToken, targetToken };
        try {
            const response = await axios_1.default.post(exports.RELAYER_FEE_API_URL, request);
            return BigInt(response.data.fee);
        }
        catch (e) {
            if (axios_1.default.isAxiosError(e)) {
                throw new Error(`Error getting relayer fee: ${e.response?.statusText}`);
            }
            throw e;
        }
    }
}
exports.PorticoApi = PorticoApi;
//# sourceMappingURL=porticoApi.js.map