"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvmPorticoBridge = void 0;
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const connect_sdk_evm_1 = require("@wormhole-foundation/connect-sdk-evm");
const ethers_1 = require("ethers");
const abis_1 = require("./abis");
const consts_1 = require("./consts");
const utils_1 = require("./utils");
const connect_sdk_evm_core_1 = require("@wormhole-foundation/connect-sdk-evm-core");
require("@wormhole-foundation/connect-sdk-evm-tokenbridge");
const porticoApi_1 = require("./porticoApi");
class EvmPorticoBridge {
    network;
    chain;
    provider;
    contracts;
    core;
    chainId;
    porticoAddress;
    uniswapAddress;
    porticoContract;
    uniswapContract;
    constructor(network, chain, provider, contracts, core) {
        this.network = network;
        this.chain = chain;
        this.provider = provider;
        this.contracts = contracts;
        this.core = core;
        const { portico, uniswapQuoterV2 } = (0, consts_1.getContracts)(chain);
        this.porticoAddress = portico;
        this.uniswapAddress = uniswapQuoterV2;
        this.chainId = connect_sdk_1.nativeChainIds.networkChainToNativeChainId.get(network, chain);
        this.porticoContract = new ethers_1.ethers.Contract(this.porticoAddress, abis_1.porticoAbi.fragments, this.provider);
        this.uniswapContract = new ethers_1.ethers.Contract(this.uniswapAddress, abis_1.uniswapQuoterV2Abi.fragments, this.provider);
    }
    static async fromRpc(provider, config) {
        const [network, chain] = await connect_sdk_evm_1.EvmPlatform.chainFromRpc(provider);
        const core = await connect_sdk_evm_core_1.EvmWormholeCore.fromRpc(provider, config);
        const conf = config[chain];
        if (conf.network !== network)
            throw new Error(`Network mismatch: ${conf.network} != ${network}`);
        return new EvmPorticoBridge(network, chain, provider, conf.contracts, core);
    }
    async *transfer(sender, receiver, token, amount, destToken, swapAmounts, relayerFee) {
        const { minAmountStart, minAmountFinish } = swapAmounts;
        if (minAmountStart === 0n)
            throw new Error('Invalid min swap amount');
        if (minAmountFinish === 0n)
            throw new Error('Invalid min swap amount');
        const senderAddress = new connect_sdk_evm_1.EvmAddress(sender).toString();
        const receiverAddress = (0, connect_sdk_1.canonicalAddress)(receiver);
        const [isStartTokenNative, startToken] = (0, connect_sdk_1.resolveWrappedToken)(this.network, this.chain, token);
        const [isFinalTokenNative, finalToken] = (0, connect_sdk_1.resolveWrappedToken)(this.network, receiver.chain, destToken);
        const startTokenAddress = (0, connect_sdk_1.canonicalAddress)(startToken);
        const finalTokenAddress = (0, connect_sdk_1.canonicalAddress)(finalToken);
        const destinationPorticoAddress = (0, consts_1.getContracts)(receiver.chain).portico;
        const receiverChainId = connect_sdk_1.nativeChainIds.networkChainToNativeChainId.get(this.network, receiver.chain);
        // Create the order
        const createOrderResult = await porticoApi_1.PorticoApi.createOrder({
            startingChainId: Number(this.chainId),
            startingToken: startTokenAddress,
            startingTokenAmount: amount.toString(),
            destinationToken: finalTokenAddress,
            destinationAddress: receiverAddress,
            destinationChainId: Number(receiverChainId),
            feeTierStart: consts_1.FEE_TIER,
            feeTierEnd: consts_1.FEE_TIER,
            minAmountStart: minAmountStart.toString(),
            minAmountEnd: minAmountFinish.toString(),
            bridgeNonce: new Date().valueOf(),
            shouldWrapNative: isStartTokenNative,
            shouldUnwrapNative: isFinalTokenNative,
            porticoAddress: this.porticoAddress,
            relayerFee: relayerFee.toString(),
            destinationPorticoAddress,
        });
        // Approve the token if necessary
        if (!isStartTokenNative)
            yield* this.approve(startTokenAddress, senderAddress, amount, this.porticoAddress);
        const messageFee = await this.core.getMessageFee();
        const tx = {
            to: this.porticoAddress,
            data: createOrderResult.transactionData,
            value: messageFee + (isStartTokenNative ? amount : 0n),
        };
        yield this.createUnsignedTransaction((0, connect_sdk_evm_1.addFrom)(tx, senderAddress), 'PorticoBridge.Transfer');
    }
    async *redeem(vaa, signer) {
        const txReq = await this.porticoContract
            .getFunction('receiveMessageAndSwap')
            .populateTransaction((0, connect_sdk_1.serialize)(vaa));
        yield this.createUnsignedTransaction((0, connect_sdk_evm_1.addFrom)(txReq, signer.address()), 'PorticoBridge.Redeem');
    }
    async quoteSwap(input, output, amount) {
        const [, inputTokenId] = (0, connect_sdk_1.resolveWrappedToken)(this.network, this.chain, input);
        const [, outputTokenId] = (0, connect_sdk_1.resolveWrappedToken)(this.network, this.chain, output);
        const inputAddress = (0, connect_sdk_1.canonicalAddress)(inputTokenId);
        const outputAddress = (0, connect_sdk_1.canonicalAddress)(outputTokenId);
        if ((0, utils_1.isEqualCaseInsensitive)(inputAddress, outputAddress))
            return amount;
        const result = await this.uniswapContract
            .getFunction('quoteExactInputSingle')
            .staticCall([inputAddress, outputAddress, amount, consts_1.FEE_TIER, 0]);
        return result[0];
    }
    async quoteRelay(token, destination) {
        const [, startToken] = (0, connect_sdk_1.resolveWrappedToken)(this.network, this.chain, token);
        return await porticoApi_1.PorticoApi.quoteRelayer(startToken, destination);
    }
    async *approve(token, senderAddr, amount, contract) {
        const tokenContract = connect_sdk_evm_1.EvmPlatform.getTokenImplementation(this.provider, token);
        const allowance = await tokenContract.allowance(senderAddr, contract);
        if (allowance < amount) {
            const txReq = await tokenContract.approve.populateTransaction(contract, amount);
            yield this.createUnsignedTransaction((0, connect_sdk_evm_1.addFrom)(txReq, senderAddr), 'PorticoBridge.Approve');
        }
    }
    createUnsignedTransaction(txReq, description) {
        return new connect_sdk_evm_1.EvmUnsignedTransaction((0, connect_sdk_evm_1.addChainId)(txReq, this.chainId), this.network, this.chain, description, false);
    }
}
exports.EvmPorticoBridge = EvmPorticoBridge;
//# sourceMappingURL=porticoBridge.js.map