import { canonicalAddress, nativeChainIds, resolveWrappedToken, serialize, } from '@wormhole-foundation/connect-sdk';
import { EvmAddress, EvmPlatform, EvmUnsignedTransaction, addChainId, addFrom, } from '@wormhole-foundation/connect-sdk-evm';
import { ethers } from 'ethers';
import { porticoAbi, uniswapQuoterV2Abi } from './abis';
import { FEE_TIER, getContracts } from './consts';
import { isEqualCaseInsensitive } from './utils';
import { EvmWormholeCore } from '@wormhole-foundation/connect-sdk-evm-core';
import '@wormhole-foundation/connect-sdk-evm-tokenbridge';
import { PorticoApi } from './porticoApi';
export class EvmPorticoBridge {
    network;
    chain;
    provider;
    contracts;
    core;
    chainId;
    porticoAddress;
    uniswapAddress;
    porticoContract;
    uniswapContract;
    constructor(network, chain, provider, contracts, core) {
        this.network = network;
        this.chain = chain;
        this.provider = provider;
        this.contracts = contracts;
        this.core = core;
        const { portico, uniswapQuoterV2 } = getContracts(chain);
        this.porticoAddress = portico;
        this.uniswapAddress = uniswapQuoterV2;
        this.chainId = nativeChainIds.networkChainToNativeChainId.get(network, chain);
        this.porticoContract = new ethers.Contract(this.porticoAddress, porticoAbi.fragments, this.provider);
        this.uniswapContract = new ethers.Contract(this.uniswapAddress, uniswapQuoterV2Abi.fragments, this.provider);
    }
    static async fromRpc(provider, config) {
        const [network, chain] = await EvmPlatform.chainFromRpc(provider);
        const core = await EvmWormholeCore.fromRpc(provider, config);
        const conf = config[chain];
        if (conf.network !== network)
            throw new Error(`Network mismatch: ${conf.network} != ${network}`);
        return new EvmPorticoBridge(network, chain, provider, conf.contracts, core);
    }
    async *transfer(sender, receiver, token, amount, destToken, swapAmounts, relayerFee) {
        const { minAmountStart, minAmountFinish } = swapAmounts;
        if (minAmountStart === 0n)
            throw new Error('Invalid min swap amount');
        if (minAmountFinish === 0n)
            throw new Error('Invalid min swap amount');
        const senderAddress = new EvmAddress(sender).toString();
        const receiverAddress = canonicalAddress(receiver);
        const [isStartTokenNative, startToken] = resolveWrappedToken(this.network, this.chain, token);
        const [isFinalTokenNative, finalToken] = resolveWrappedToken(this.network, receiver.chain, destToken);
        const startTokenAddress = canonicalAddress(startToken);
        const finalTokenAddress = canonicalAddress(finalToken);
        const destinationPorticoAddress = getContracts(receiver.chain).portico;
        const receiverChainId = nativeChainIds.networkChainToNativeChainId.get(this.network, receiver.chain);
        // Create the order
        const createOrderResult = await PorticoApi.createOrder({
            startingChainId: Number(this.chainId),
            startingToken: startTokenAddress,
            startingTokenAmount: amount.toString(),
            destinationToken: finalTokenAddress,
            destinationAddress: receiverAddress,
            destinationChainId: Number(receiverChainId),
            feeTierStart: FEE_TIER,
            feeTierEnd: FEE_TIER,
            minAmountStart: minAmountStart.toString(),
            minAmountEnd: minAmountFinish.toString(),
            bridgeNonce: new Date().valueOf(),
            shouldWrapNative: isStartTokenNative,
            shouldUnwrapNative: isFinalTokenNative,
            porticoAddress: this.porticoAddress,
            relayerFee: relayerFee.toString(),
            destinationPorticoAddress,
        });
        // Approve the token if necessary
        if (!isStartTokenNative)
            yield* this.approve(startTokenAddress, senderAddress, amount, this.porticoAddress);
        const messageFee = await this.core.getMessageFee();
        const tx = {
            to: this.porticoAddress,
            data: createOrderResult.transactionData,
            value: messageFee + (isStartTokenNative ? amount : 0n),
        };
        yield this.createUnsignedTransaction(addFrom(tx, senderAddress), 'PorticoBridge.Transfer');
    }
    async *redeem(vaa, signer) {
        const txReq = await this.porticoContract
            .getFunction('receiveMessageAndSwap')
            .populateTransaction(serialize(vaa));
        yield this.createUnsignedTransaction(addFrom(txReq, signer.address()), 'PorticoBridge.Redeem');
    }
    async quoteSwap(input, output, amount) {
        const [, inputTokenId] = resolveWrappedToken(this.network, this.chain, input);
        const [, outputTokenId] = resolveWrappedToken(this.network, this.chain, output);
        const inputAddress = canonicalAddress(inputTokenId);
        const outputAddress = canonicalAddress(outputTokenId);
        if (isEqualCaseInsensitive(inputAddress, outputAddress))
            return amount;
        const result = await this.uniswapContract
            .getFunction('quoteExactInputSingle')
            .staticCall([inputAddress, outputAddress, amount, FEE_TIER, 0]);
        return result[0];
    }
    async quoteRelay(token, destination) {
        const [, startToken] = resolveWrappedToken(this.network, this.chain, token);
        return await PorticoApi.quoteRelayer(startToken, destination);
    }
    async *approve(token, senderAddr, amount, contract) {
        const tokenContract = EvmPlatform.getTokenImplementation(this.provider, token);
        const allowance = await tokenContract.allowance(senderAddr, contract);
        if (allowance < amount) {
            const txReq = await tokenContract.approve.populateTransaction(contract, amount);
            yield this.createUnsignedTransaction(addFrom(txReq, senderAddr), 'PorticoBridge.Approve');
        }
    }
    createUnsignedTransaction(txReq, description) {
        return new EvmUnsignedTransaction(addChainId(txReq, this.chainId), this.network, this.chain, description, false);
    }
}
//# sourceMappingURL=porticoBridge.js.map